pub struct TopoEdge {
pub id: EdgeIndex,
pub from: NodeId,
pub to: NodeId,
pub edge_id: EdgeId,
}Expand description
Represents an edge in the TopologyGraph.
Each edge is identified by an EdgeIndex (which represents the edge’s position in the petgraph),
a from and to NodeId (representing the nodes that the edge connects),
and an EdgeId (a custom identifier).
Fields
id: EdgeIndex- The index of the edge in the petgraph.from: NodeId- The custom identifier of the node where the edge originates.to: NodeId- The custom identifier of the node where the edge ends.edge_id: EdgeId- The custom identifier of the edge.
Fields
id: EdgeIndexThe index of the edge in the petgraph.
from: NodeIdThe custom identifier of the node where the edge originates.
to: NodeIdThe custom identifier of the node where the edge ends.
edge_id: EdgeIdThe custom identifier of the edge.